@@ -3,10 +3,15 @@ from imports import *
 from plotly_graphs import *
 from plotly_css import *
 
-external_stylesheets = ['https://codepen.io/rab657/pen/LYpKraq.css',
+external_stylesheets = ['https://raw.githubusercontent.com/rab657/explainx/master/explainx.css',
+
                             {
                                 'href': 'https://stackpath.bootstrapcdn.com/bootstrap/4.5.0/css/bootstrap.min.css',
                                 'rel': 'stylesheet'
+                            }, 
+                            {
+                                'href':'https://fonts.googleapis.com/css?family=Montserrat',
+                                'rel' :'stylesheet'
                             }
                             ]
 
